@@ -116,6 +116,8 @@ char const *eventtype_to_string (UINT32 event_type) {
         return "EV_EFI_HANDOFF_TABLES2";
     case EV_EFI_VARIABLE_BOOT2:
         return "EV_EFI_VARIABLE_BOOT2";
+    case EV_EFI_HCRTM_EVENT:
+        return "EV_EFI_HCRTM_EVENT";
     case EV_EFI_VARIABLE_AUTHORITY:
         return "EV_EFI_VARIABLE_AUTHORITY";
     default:
@@ -255,6 +257,18 @@ static bool yaml_uefi_post_code(const TCG_EVENT2* const event) {
     }
     return true;
 }
+
+static bool yaml_uefi_hcrtm(const TCG_EVENT2* const event) {
+
+    const size_t len = event->EventSize;
+
+    const char* const data = (const char *) event->Event;
+    tpm2_tool_output("  Event: |-\n"
+                     "    %.*s\n", (int) len, data);
+
+    return true;
+}
+
 /*
  * Parses Device Path field using the efivar library if present, otherwise,
  * print the field in raw byte format
@@ -277,7 +291,8 @@ char *yaml_devicepath(BYTE* dp, UINT64 dp_len) {
         return NULL;
     }
   
-    ret = efidp_format_device_path((unsigned char *)text_path,
+    /* The void* cast is a hack to support efivar versions < 38 */
+    ret = efidp_format_device_path((void *)text_path,
             text_path_len, (const_efidp)dp, dp_len);
     if (ret < 0) {
         free(text_path);
@@ -296,9 +311,9 @@ char *yaml_devicepath(BYTE* dp, UINT64 dp_len) {
  *
  * We need to emit YAML with some rules:
  *
- *  - No leading ' ' without quoting it
+ *  - No leading ' ' or \t without escaping it
  *  - Escape non-printable ascii chars
- *  - Double quotes if using escape sequences
+ *  - Double quotes to enable use of escape sequences
  *  - Valid UTF8 string
  *
  * This method will ignore the question of original data
@@ -324,8 +339,14 @@ char **yaml_split_escape_string(UINT8 const *description, size_t size)
 
     i = 0;
     do {
+        bool leadingSpace = true;
         nl = memchr(description + i, '\n', size - i);
-        len = nl ? (size_t)(nl - (description + i)) : size - i;
+        if (nl) {
+            nl++;
+            len = (size_t)(nl - (description + i));
+        } else {
+            len = size - i;
+        }
 
         tmp = realloc(lines, sizeof(char *) * (nlines + 2));
         if (!tmp) {
@@ -337,62 +358,77 @@ char **yaml_split_escape_string(UINT8 const *description, size_t size)
         lines[nlines + 1] = NULL;
         k = 0;
 
-        /* Worst case: every byte needs escaping, plus start/end quotes, plus nul */
-        lines[nlines] = calloc(1, (len * 2) + 2 + 1);
+        /* Worst case: every byte is a space that needs escaping to a 4 byte
+         * sequence, plus a line continuation, plus nul. We're overallocating
+         * here, but the caller is going to free all these strings immediately
+         * after printing, so this is fairly harmless */
+        lines[nlines] = calloc(1, (len * 4) + 1 + 1);
         if (!lines[nlines]) {
             LOG_ERR("failed to allocate memory for escaped string: %s\n",
                     strerror(errno));
             goto error;
         }
 
-        lines[nlines][k++] = '"';
         for (j = i; j < (i + len); j++) {
-            char escape = '\0';
+            const char *escape = NULL;
 
             switch (description[j]) {
             case '\0':
-              escape = '0';
+              escape = "\\0";
               break;
             case '\a':
-              escape = 'a';
+              escape = "\\a";
               break;
             case '\b':
-              escape = 'b';
+              escape = "\\b";
               break;
             case '\t':
-              escape = 't';
+              if (leadingSpace)
+                  escape = "\\t";
               break;
             case '\v':
-              escape = 'v';
+              escape = "\\v";
               break;
             case '\f':
-              escape = 'f';
+              escape = "\\f";
+              break;
+            case '\n':
+              escape = "\\n";
               break;
             case '\r':
-              escape = 'r';
+              escape = "\\r";
               break;
             case '\e':
-              escape = 'e';
-              break;
-            case '\'':
-              escape = '\'';
+              escape = "\\e";
               break;
             case '\\':
-              escape = '\\';
+              escape = "\\\\";
+              break;
+            case '"':
+              escape = "\\\"";
+              break;
+            case ' ':
+              if (leadingSpace)
+                  escape = "\\x20";
               break;
             }
 
-            if (escape == '\0') {
+            if (description[j] != ' ' &&
+                description[j] != '\t') {
+                leadingSpace = false;
+            }
+
+            if (escape == NULL) {
                 lines[nlines][k++] = description[j];
             } else {
-                lines[nlines][k++] = '\\';
-                lines[nlines][k++] = escape;
+                while (*escape) {
+                    lines[nlines][k++] = *escape;
+                    escape++;
+                }
             }
         }
-        lines[nlines][k++] = '"';
-
         nlines++;
-        i += len + 1;
+        i += len;
     } while (i < size);
 
     return lines;
@@ -405,6 +441,37 @@ char **yaml_split_escape_string(UINT8 const *description, size_t size)
     return NULL;
 }
 
+static bool yaml_split_print_string(const char *indent,
+                                    const char *field,
+                                    UINT8 const *description, size_t size)
+{
+    char **lines = NULL;
+    size_t i;
+    tpm2_tool_output("%s%s: \"", indent, field);
+
+    lines = yaml_split_escape_string(description, size);
+    if (!lines) {
+        return false;
+    }
+
+    for (i = 0; lines[i] != NULL; i++) {
+        if (i == 0)
+            tpm2_tool_output("%s", lines[i]);
+        else
+            tpm2_tool_output("%s  %s", indent, lines[i]);
+
+        if (lines[i+1] == NULL) {
+            tpm2_tool_output("\"\n");
+        } else {
+            tpm2_tool_output("\\\n");
+        }
+
+        free(lines[i]);
+    }
+    free(lines);
+    return true;
+}
+
 /*
  * TCG PC Client FPF section 9.2.6
  * The tpm2_eventlog module validates the event structure but nothing within
@@ -604,24 +671,12 @@ static bool yaml_uefi_var(UEFI_VARIABLE_DATA *data, size_t size, UINT32 type,
                 return true;
             } else if (strlen(ret) == NAME_SBATLEVEL_LEN && strncmp(ret, NAME_SBATLEVEL, NAME_SBATLEVEL_LEN) == 0)  {
                 free(ret);
-                tpm2_tool_output("    VariableData:\n"
-                                 "      String: |-\n");
+                tpm2_tool_output("    VariableData:\n");
 
                 UINT8 *description = (UINT8 *)&data->UnicodeName[
                     data->UnicodeNameLength];
-                char **lines = NULL;
-                lines = yaml_split_escape_string(description, data->VariableDataLength);
-                if (!lines) {
-                    return false;
-                }
-
-                size_t i;
-                for (i = 0; lines[i] != NULL; i++) {
-                    tpm2_tool_output("        %s\n", lines[i]);
-                    free(lines[i]);
-                }
-                free(lines);
-                return true;
+                return yaml_split_print_string("      ", "String",
+                                               description, data->VariableDataLength);
             }
         } else if (type == EV_EFI_VARIABLE_BOOT || type == EV_EFI_VARIABLE_BOOT2) {
             if ((strlen(ret) == NAME_BOOTORDER_LEN && strncmp(ret, NAME_BOOTORDER, NAME_BOOTORDER_LEN) == 0)) {
@@ -763,23 +818,10 @@ bool yaml_uefi_action(UINT8 const *action, size_t size) {
  * the loading of grub, kernel, and initrd images.
  */
 bool yaml_ipl(UINT8 const *description, size_t size) {
-    char **lines = NULL;
-    size_t i;
-    tpm2_tool_output("  Event:\n"
-                     "    String: |-\n");
+    tpm2_tool_output("  Event:\n");
 
-    lines = yaml_split_escape_string(description, size);
-    if (!lines) {
-        return false;
-    }
-
-    for (i = 0; lines[i] != NULL; i++) {
-        tpm2_tool_output("      %s\n", lines[i]);
-        free(lines[i]);
-    }
-    free(lines);
-
-    return true;
+    return yaml_split_print_string("    ", "String",
+                                   description, size);
 }
 /* TCG PC Client PFP section 9.2.3 */
 bool yaml_uefi_image_load(UEFI_IMAGE_LOAD_EVENT *data, size_t size) {
@@ -960,6 +1002,8 @@ bool yaml_event2data(TCG_EVENT2 const *event, UINT32 type, uint32_t eventlog_ver
                         event->EventSize, eventlog_version);
     case EV_NO_ACTION:
         return yaml_no_action((EV_NO_ACTION_STRUCT*)event->Event, event->EventSize, eventlog_version);
+    case EV_EFI_HCRTM_EVENT:
+        return yaml_uefi_hcrtm(event);
     default:
         bytes_to_str(event->Event, event->EventSize, hexstr, sizeof(hexstr));
         tpm2_tool_output("  Event: \"%s\"\n", hexstr);
