@@ -119,4 +119,14 @@ tpm2 flushcontext enc_session.ctx
 tpm2_create -C primary.ctx -u obj.pub -r obj.priv -f pem -o obj.pem
 openssl rsa -noout -text -inform PEM -in obj.pem -pubin
 
+# Test policy from hash input
+
+expected_dgst="fdb1c1e5ba81e95f2db8db6ed7627e9b01658e80df7f33220bd3638f98ad2d5f"
+tpm2 createprimary -c primary.ctx
+tpm2 create -C primary.ctx -u key.pub -r key.priv -L "$(expected_digest)"
+tpm2 load -C primary.ctx -u key.pub -r key.priv -c key.ctx
+got_digest="$(tpm2 readpublic -c key.ctx | grep "authorization policy" | cut -d ' ' -f3-)"
+test "$expected_digest" == "$got_digest"
+
+
 exit 0
