@@ -1,6 +1,6 @@
 # SPDX-License-Identifier: BSD-3-Clause
 
-###this script use for test the implementation tpm2_dictionarylockout
+###this script use for test the implementation tpm2 dictionarylockout
 
 source helpers.sh
 
@@ -14,11 +14,11 @@ trap cleanup EXIT
 
 start_up
 
-tpm2_dictionarylockout -Q -V -c &>/dev/null
+tpm2 dictionarylockout -Q -V -c &>/dev/null
 
-tpm2_dictionarylockout -s -n 5 -t 6 -l 7
+tpm2 dictionarylockout -s -n 5 -t 6 -l 7
 
-tpm2_getcap properties-variable > $out
+tpm2 getcap properties-variable > $out
 v=$(yaml_get_kv "$out" "TPM2_PT_MAX_AUTH_FAIL")
 if [ $v -ne 5 ];then
   echo "Failure: setting up the number of allowed tries in the lockout \
