@@ -1,40 +1,13 @@
-//**********************************************************************;
-// Copyright (c) 2016, Intel Corporation
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice,
-// this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright notice,
-// this list of conditions and the following disclaimer in the documentation
-// and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-// THE POSSIBILITY OF SUCH DAMAGE.
-//**********************************************************************;
+/* SPDX-License-Identifier: BSD-3-Clause */
+
 #include <stdarg.h>
 #include <stdbool.h>
-#include <stddef.h>
-#include <setjmp.h>
 #include <stdlib.h>
 
+#include <setjmp.h>
 #include <cmocka.h>
-#include <tss2/tss2_sys.h>
 
 #include "tpm2_attr_util.h"
-#include "tpm2_util.h"
 
 #define test_nv_strtoattr_get(set) \
     cmocka_unit_test(test_tpm2_attr_util_nv_strtoattr_##set)
@@ -44,14 +17,12 @@
         \
         (void)state; \
     \
-        TPMA_NV nvattrs = set; \
-    \
+        TPMA_NV nvattrs = 0; \
         /* make mutable strings for strtok_r */ \
         char arg[] = argstr; \
         bool res = tpm2_attr_util_nv_strtoattr(arg, &nvattrs); \
         assert_true(res); \
         assert_true(nvattrs == set); \
-        assert_true(nvattrs == set); \
     }
 
 nv_single_item_test("authread", TPMA_NV_AUTHREAD);
@@ -76,10 +47,6 @@ nv_single_item_test("writelocked", TPMA_NV_WRITELOCKED);
 nv_single_item_test("write_stclear", TPMA_NV_WRITE_STCLEAR);
 nv_single_item_test("written", TPMA_NV_WRITTEN);
 
-static inline UINT32 from_nt(TPMA_NV nvattrs) {
-    return ((nvattrs & TPMA_NV_TPM2_NT_MASK) >> TPMA_NV_TPM2_NT_SHIFT);
-}
-
 static void test_tpm2_attr_util_nv_strtoattr_nt_good(void **state) {
     (void) state;
 
@@ -88,7 +55,8 @@ static void test_tpm2_attr_util_nv_strtoattr_nt_good(void **state) {
     char arg[] = "nt=0x1";
     bool res = tpm2_attr_util_nv_strtoattr(arg, &nvattrs);
     assert_true(res);
-    assert_true(from_nt(nvattrs) == 0x1);
+    assert_true((nvattrs & TPMA_NV_TPM2_NT_MASK) >> TPMA_NV_TPM2_NT_SHIFT ==
+        0x1);
 }
 
 static void test_tpm2_attr_util_nv_strtoattr_nt_bad(void **state) {
@@ -137,7 +105,8 @@ static void test_tpm2_attr_util_nv_strtoattr_multiple_good(void **state) {
     char arg[] = "authread|authwrite|nt=0x4";
     bool res = tpm2_attr_util_nv_strtoattr(arg, &nvattrs);
     assert_true(res);
-    assert_true(from_nt(nvattrs) == 0x4);
+    assert_true((nvattrs & TPMA_NV_TPM2_NT_MASK) >> TPMA_NV_TPM2_NT_SHIFT ==
+        0x4);
     assert_true(nvattrs & TPMA_NV_AUTHREAD);
     assert_true(nvattrs & TPMA_NV_AUTHWRITE);
 }
@@ -159,7 +128,7 @@ static void test_tpm2_attr_util_nv_strtoattr_token_unknown(void **state) {
     res = tpm2_attr_util_nv_strtoattr(arg2, &nvattrs);
     assert_false(res);
 
-    /* should be interprested as the whole thing, no = */
+    /* should be interpreted as the whole thing, no = */
     char arg3[] = "nt:0x4";
     res = tpm2_attr_util_nv_strtoattr(arg3, &nvattrs);
     assert_false(res);
@@ -238,19 +207,18 @@ test_nv_attrtostr(0xFFFFFFFF, NV_ALL_FIELDS);
     }
 
 test_nv_attrtostr_compound(stclear_ppwrite,
-        TPMA_NV_WRITE_STCLEAR|TPMA_NV_PPWRITE,
-        "ppwrite|write_stclear")
+        TPMA_NV_WRITE_STCLEAR | TPMA_NV_PPWRITE, "ppwrite|write_stclear")
 test_nv_attrtostr_compound(stclear_ppwrite_0x30,
-        TPMA_NV_WRITE_STCLEAR|TPMA_NV_PPWRITE|0x30,
+        TPMA_NV_WRITE_STCLEAR | TPMA_NV_PPWRITE | 0x30,
         "ppwrite|nt=0x3|write_stclear")
-test_nv_attrtostr_compound(platformcreate_owneread_nt_0x90_0x20000,
-        TPMA_NV_PLATFORMCREATE|TPMA_NV_AUTHWRITE|0x90|0x200000,
+test_nv_attrtostr_compound(platformcreate_ownerread_nt_0x90_0x20000,
+        TPMA_NV_PLATFORMCREATE | TPMA_NV_AUTHWRITE | 0x90 | 0x200000,
         "authwrite|nt=0x9|<reserved(21)>|platformcreate")
 
 /*
  * TPMA_OBJECT Tests
  */
-#define obj_single_item_test(argstr, index, set) \
+#define obj_single_item_test(argstr, set) \
     static void test_tpm2_attr_util_nv_strtoattr_##set(void **state) { \
         \
         (void)state; \
@@ -260,23 +228,23 @@ test_nv_attrtostr_compound(platformcreate_owneread_nt_0x90_0x20000,
         char arg[] = argstr; \
         bool res = tpm2_attr_util_obj_strtoattr(arg, &objattrs); \
         assert_true(res); \
-        assert_true(objattrs == set); \
+        assert_true(objattrs & set); \
     }
 
 #define test_obj_strtoattr_get(set) \
     cmocka_unit_test(test_tpm2_attr_util_nv_strtoattr_##set)
 
-obj_single_item_test("fixedtpm", fixedTPM, TPMA_OBJECT_FIXEDTPM);
-obj_single_item_test("stclear", stClear, TPMA_OBJECT_STCLEAR);
-obj_single_item_test("fixedparent", fixedParent, TPMA_OBJECT_FIXEDPARENT);
-obj_single_item_test("sensitivedataorigin", sensitiveDataOrigin, TPMA_OBJECT_SENSITIVEDATAORIGIN);
-obj_single_item_test("userwithauth", userWithAuth, TPMA_OBJECT_USERWITHAUTH);
-obj_single_item_test("adminwithpolicy", adminWithPolicy, TPMA_OBJECT_ADMINWITHPOLICY);
-obj_single_item_test("noda", noDA, TPMA_OBJECT_NODA);
-obj_single_item_test("encryptedduplication", encryptedDuplication, TPMA_OBJECT_ENCRYPTEDDUPLICATION);
-obj_single_item_test("restricted", restricted, TPMA_OBJECT_RESTRICTED);
-obj_single_item_test("decrypt", decrypt, TPMA_OBJECT_DECRYPT);
-obj_single_item_test("sign", sign, TPMA_OBJECT_SIGN_ENCRYPT);
+obj_single_item_test("fixedtpm", TPMA_OBJECT_FIXEDTPM);
+obj_single_item_test("stclear", TPMA_OBJECT_STCLEAR);
+obj_single_item_test("fixedparent", TPMA_OBJECT_FIXEDPARENT);
+obj_single_item_test("sensitivedataorigin", TPMA_OBJECT_SENSITIVEDATAORIGIN);
+obj_single_item_test("userwithauth", TPMA_OBJECT_USERWITHAUTH);
+obj_single_item_test("adminwithpolicy", TPMA_OBJECT_ADMINWITHPOLICY);
+obj_single_item_test("noda", TPMA_OBJECT_NODA);
+obj_single_item_test("encryptedduplication", TPMA_OBJECT_ENCRYPTEDDUPLICATION);
+obj_single_item_test("restricted", TPMA_OBJECT_RESTRICTED);
+obj_single_item_test("decrypt", TPMA_OBJECT_DECRYPT);
+obj_single_item_test("sign", TPMA_OBJECT_SIGN_ENCRYPT);
 
 #define OBJ_ALL_FIELDS \
         "<reserved(0)>|fixedtpm|stclear|<reserved(3)>|fixedparent" \
@@ -285,7 +253,7 @@ obj_single_item_test("sign", sign, TPMA_OBJECT_SIGN_ENCRYPT);
         "<reserved(13)>|<reserved(14)>|<reserved(15)>|restricted|decrypt|" \
         "sign|<reserved(19)>|<reserved(20)>|<reserved(21)>|<reserved(22)>|" \
         "<reserved(23)>|<reserved(24)>|<reserved(25)>|<reserved(26)>|" \
-		"<reserved(27)>|<reserved(28)>|<reserved(29)>|<reserved(30)>|" \
+        "<reserved(27)>|<reserved(28)>|<reserved(29)>|<reserved(30)>|" \
         "<reserved(31)>"
 
 #define test_obj_attrtostr(value, expected) \
@@ -293,14 +261,13 @@ obj_single_item_test("sign", sign, TPMA_OBJECT_SIGN_ENCRYPT);
     \
         (void) state; \
     \
-	    TPMA_OBJECT attrs = value; \
+        TPMA_OBJECT attrs = value; \
         char *str = tpm2_attr_util_obj_attrtostr(attrs); \
         assert_string_equal(str, expected); \
     \
         free(str); \
     }
 
-
 #define test_obj_attrtostr_get(value) \
         cmocka_unit_test(test_tpm2_obj_util_attrtostr_##value)
 
@@ -318,6 +285,16 @@ test_obj_attrtostr(TPMA_OBJECT_RESTRICTED, "restricted");
 test_obj_attrtostr(TPMA_OBJECT_DECRYPT, "decrypt");
 test_obj_attrtostr(TPMA_OBJECT_SIGN_ENCRYPT, "sign");
 
+test_obj_attrtostr(TPMA_OBJECT_RESERVED1_MASK, "<reserved(0)>");
+test_obj_attrtostr(TPMA_OBJECT_RESERVED2_MASK, "<reserved(3)>");
+test_obj_attrtostr(TPMA_OBJECT_RESERVED3_MASK, "<reserved(8)>|<reserved(9)>");
+test_obj_attrtostr(TPMA_OBJECT_RESERVED4_MASK, "<reserved(12)>|<reserved(13)>|" \
+        "<reserved(14)>|<reserved(15)>");
+test_obj_attrtostr(TPMA_OBJECT_RESERVED5_MASK, "<reserved(19)>|<reserved(20)>|" \
+        "<reserved(21)>|<reserved(22)>|<reserved(23)>|<reserved(24)>|" \
+        "<reserved(25)>|<reserved(26)>|<reserved(27)>|<reserved(28)>|" \
+        "<reserved(29)>|<reserved(30)>|<reserved(31)>");
+
 static void test_tpm2_attr_util_obj_strtoattr_multiple_good(void **state) {
     (void) state;
 
@@ -331,20 +308,21 @@ static void test_tpm2_attr_util_obj_strtoattr_multiple_good(void **state) {
     assert_true(objattrs & TPMA_OBJECT_NODA);
 
     assert_int_equal(objattrs,
-            TPMA_OBJECT_SIGN_ENCRYPT|TPMA_OBJECT_NODA|TPMA_OBJECT_ADMINWITHPOLICY);
+            TPMA_OBJECT_SIGN_ENCRYPT | TPMA_OBJECT_NODA
+                    | TPMA_OBJECT_ADMINWITHPOLICY);
 }
 
 static void test_tpm2_attr_util_obj_strtoattr_token_unknown(void **state) {
     (void) state;
 
-    TPMA_OBJECT objattrrs = 0;
+    TPMA_OBJECT objattrs = 0;
 
-    char arg[] = "fixedtpm|noda|unkown";
-    bool res = tpm2_attr_util_obj_strtoattr(arg, &objattrrs);
+    char arg[] = "fixedtpm|noda|unknown";
+    bool res = tpm2_attr_util_obj_strtoattr(arg, &objattrs);
     assert_false(res);
 
     char arg1[] = "foo";
-    res = tpm2_attr_util_obj_strtoattr(arg1, &objattrrs);
+    res = tpm2_attr_util_obj_strtoattr(arg1, &objattrs);
     assert_false(res);
 }
 
@@ -363,6 +341,11 @@ static void test_tpm2_attr_util_obj_from_optarg_good(void **state) {
     assert_int_equal(TPMA_OBJECT_FIXEDTPM, objattrs);
 }
 
+/* link required symbol, but tpm2_tool.c declares it AND main, which
+ * we have a main below for cmocka tests.
+ */
+bool output_enabled = true;
+
 int main(int argc, char* argv[]) {
     (void) argc;
     (void) argv;
@@ -430,7 +413,7 @@ int main(int argc, char* argv[]) {
             test_nv_attrtostr_get(0x90), //nt=0x9
             test_nv_attrtostr_get(stclear_ppwrite),
             test_nv_attrtostr_get(stclear_ppwrite_0x30),
-            test_nv_attrtostr_get(platformcreate_owneread_nt_0x90_0x20000),
+            test_nv_attrtostr_get(platformcreate_ownerread_nt_0x90_0x20000),
             /* TPMA_OBJECT Tests */
 
             /* From String to Attribute value */
@@ -460,6 +443,11 @@ int main(int argc, char* argv[]) {
             test_obj_attrtostr_get(TPMA_OBJECT_RESTRICTED),
             test_obj_attrtostr_get(TPMA_OBJECT_DECRYPT),
             test_obj_attrtostr_get(TPMA_OBJECT_SIGN_ENCRYPT),
+            test_obj_attrtostr_get(TPMA_OBJECT_RESERVED1_MASK),
+            test_obj_attrtostr_get(TPMA_OBJECT_RESERVED2_MASK),
+            test_obj_attrtostr_get(TPMA_OBJECT_RESERVED3_MASK),
+            test_obj_attrtostr_get(TPMA_OBJECT_RESERVED4_MASK),
+            test_obj_attrtostr_get(TPMA_OBJECT_RESERVED5_MASK),
 
             /* compound good */
             cmocka_unit_test(test_tpm2_attr_util_obj_strtoattr_multiple_good),
