@@ -6,11 +6,8 @@
 
 #include "tools/fapi/tss2_template.h"
 
-/* needed by tpm2_util and tpm2_option functions */
-bool output_enabled = false;
-
 /* needed to conditionally free variable authValue */
-bool has_asked_for_password = false;
+static bool has_asked_for_password = false;
 
 /* Context struct used to store passed commandline parameters */
 static struct cxt {
@@ -40,7 +37,7 @@ static bool on_option(char key, char *value) {
 }
 
 /* Define possible commandline parameters */
-bool tss2_tool_onstart(tpm2_options **opts) {
+static bool tss2_tool_onstart(tpm2_options **opts) {
     struct option topts[] = {
         {"path",       required_argument, NULL, 'p'},
         {"type",       required_argument, NULL, 't'},
@@ -52,7 +49,7 @@ bool tss2_tool_onstart(tpm2_options **opts) {
 }
 
 /* Execute specific tool */
-int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
+static int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
     /* Check availability of required parameters */
     if (!ctx.keyPath) {
         fprintf (stderr, "key path missing, use --path\n");
@@ -76,12 +73,14 @@ int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
             free (ctx.authValue);
         }
         LOG_PERR ("Fapi_CreateKey", r);
-        return r;
+        return 1;
     }
 
     if(has_asked_for_password){
         free (ctx.authValue);
     }
 
-    return r;
+    return 0;
 }
+
+TSS2_TOOL_REGISTER("createkey", tss2_tool_onstart, tss2_tool_onrun, NULL)
