@@ -7,9 +7,6 @@
 
 #include "tools/fapi/tss2_template.h"
 
-/* needed by tpm2_util and tpm2_option functions */
-bool output_enabled = false;
-
 /* Context struct used to store passed command line parameters */
 static struct cxt {
     char const *nvPath;
@@ -38,7 +35,7 @@ static bool on_option(char key, char *value) {
 }
 
 /* Define possible command line parameters */
-bool tss2_tool_onstart(tpm2_options **opts) {
+static bool tss2_tool_onstart(tpm2_options **opts) {
     struct option topts[] = {
         {"nvPath"  , required_argument, NULL, 'p'},
         {"force" , no_argument      , NULL, 'f'},
@@ -50,7 +47,7 @@ bool tss2_tool_onstart(tpm2_options **opts) {
 }
 
 /* Execute specific tool */
-int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
+static int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
     /* Check availability of required parameters */
     if (!ctx.nvPath) {
         fprintf (stderr, "No NV path provided, use --nvPath\n");
@@ -84,7 +81,6 @@ int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
     /* Write returned data to file(s) */
     r = open_write_and_close (ctx.data, ctx.overwrite, data, data_len);
     if (r) {
-        LOG_PERR ("open_write_and_close data", r);
         Fapi_Free (data);
         return 1;
     }
@@ -95,13 +91,14 @@ int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
         if (r) {
             Fapi_Free (data);
             Fapi_Free (logData);
-            LOG_PERR ("open_write_and_close logData", r);
             return 1;
         }
     }
 
-    Fapi_Free (logData);
     Fapi_Free (data);
+    Fapi_Free (logData);
 
     return 0;
 }
+
+TSS2_TOOL_REGISTER("nvread", tss2_tool_onstart, tss2_tool_onrun, NULL)
