@@ -4,9 +4,6 @@
 #include <stdlib.h>
 #include "tools/fapi/tss2_template.h"
 
-/* needed by tpm2_util and tpm2_option functions */
-bool output_enabled = false;
-
 /* Context struct used to store passed command line parameters */
 static struct cxt {
     char const *path;
@@ -27,7 +24,7 @@ static bool on_option(char key, char *value) {
 }
 
 /* Define possible command line parameters */
-bool tss2_tool_onstart(tpm2_options **opts) {
+static bool tss2_tool_onstart(tpm2_options **opts) {
     struct option topts[] = {
         {"path"    , required_argument, NULL, 'p'},
         {"x509certData", required_argument, NULL, 'i'}
@@ -37,7 +34,7 @@ bool tss2_tool_onstart(tpm2_options **opts) {
 }
 
 /* Execute specific tool */
-int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
+static int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
     /* Check availability of required parameters */
     if (!ctx.path) {
         fprintf (stderr, "path missing, use --path\n");
@@ -52,7 +49,6 @@ int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
         r = open_read_and_close (ctx.x509cert, (void**)&x509certData,
             &x509certSize);
         if (r) {
-            LOG_PERR("open_read_and_close x509cert", r);
             return 1;
         }
     }
@@ -67,3 +63,5 @@ int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
     free (x509certData);
     return 0;
 }
+
+TSS2_TOOL_REGISTER("setcertificate", tss2_tool_onstart, tss2_tool_onrun, NULL)
