@@ -175,8 +175,8 @@ static tool_rc make_credential_and_save(ESYS_CONTEXT *ectx) {
     TPM2B_ENCRYPTED_SECRET *secret;
     ESYS_TR tr_handle = ESYS_TR_NONE;
 
-    tool_rc rc = tpm2_loadexternal(ectx,
-            NULL, &ctx.public, TPM2_RH_NULL, &tr_handle);
+    tool_rc rc = tpm2_loadexternal(ectx, 0, &ctx.public, TPM2_RH_NULL,
+        &tr_handle, 0, TPM2_ALG_ERROR);
     if (rc != tool_rc_success) {
         return rc;
     }
@@ -188,7 +188,7 @@ static tool_rc make_credential_and_save(ESYS_CONTEXT *ectx) {
         return rc;
     }
 
-    rc = tpm2_flush_context(ectx, tr_handle);
+    rc = tpm2_flush_context(ectx, tr_handle, NULL, TPM2_ALG_NULL);
     if (rc != tool_rc_success) {
         free(cred_blob);
         free(secret);
@@ -310,11 +310,14 @@ static void set_default_TCG_EK_template(TPMI_ALG_PUBLIC alg) {
     ctx.public.publicArea.nameAlg = TPM2_ALG_SHA256;
 }
 
-static tool_rc process_input(void) {
+static tool_rc process_input(tpm2_option_flags flags) {
 
     TPMI_ALG_PUBLIC alg = TPM2_ALG_NULL;
     if (ctx.key_type) {
-        LOG_WARN("Because **-G** is specified, assuming input encryption public key is in PEM format.");
+        if (!flags.quiet) {
+            LOG_WARN("Because **-G** is specified, assuming input encryption "
+                     "public key is in PEM format.");
+        }
         alg = tpm2_alg_util_from_optarg(ctx.key_type,
             tpm2_alg_util_flags_asymmetric);
         if (alg == TPM2_ALG_ERROR ||
@@ -325,8 +328,9 @@ static tool_rc process_input(void) {
     }
 
     if (ctx.public_key_path) {
-        bool result = tpm2_openssl_load_public(ctx.public_key_path, alg,
-            &ctx.public);
+        bool result = alg != TPM2_ALG_NULL ?
+            tpm2_openssl_load_public(ctx.public_key_path, alg,
+            &ctx.public) : files_load_public(ctx.public_key_path, &ctx.public);
         if (!result) {
             return tool_rc_general_error;
         }
@@ -361,6 +365,16 @@ static tool_rc process_input(void) {
         return tool_rc_general_error;
     }
 
+    /*
+     * If input was read from stdin, check if a larger data set was specified
+     * and error out.
+     */
+    if (ctx.credential.size > TPM2_SHA512_DIGEST_SIZE) {
+        LOG_ERR("Size is larger than buffer, got %d expected less than or equal"
+        "to %d", ctx.credential.size, TPM2_SHA512_DIGEST_SIZE);
+        return tool_rc_general_error;
+    }
+
     return tool_rc_success;
 }
 
@@ -368,7 +382,7 @@ static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
 
     UNUSED(flags);
 
-    tool_rc rc = process_input();
+    tool_rc rc = process_input(flags);
     if (rc != tool_rc_success) {
         return rc;
     }
