@@ -53,7 +53,7 @@ static bool on_option(char key, char *value) {
     return result;
 }
 
-bool tpm2_tool_onstart(tpm2_options **opts) {
+static bool tpm2_tool_onstart(tpm2_options **opts) {
 
     static struct option topts[] = {
         { "policy",        required_argument, NULL, 'L' },
@@ -67,7 +67,7 @@ bool tpm2_tool_onstart(tpm2_options **opts) {
     return *opts != NULL;
 }
 
-bool is_input_option_args_valid(void) {
+static bool is_input_option_args_valid(void) {
 
     if (!ctx.session_file_path) {
         LOG_ERR("Must specify -S session file.");
@@ -81,7 +81,7 @@ bool is_input_option_args_valid(void) {
     return true;
 }
 
-tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
+static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
 
     UNUSED(flags);
 
@@ -105,7 +105,10 @@ tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
     return tpm2_policy_tool_finish(ectx, ctx.session, ctx.policy_digest_file_path);
 }
 
-tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
+static tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
     UNUSED(ectx);
     return tpm2_session_close(&ctx.session);
 }
+
+// Register this tool with tpm2_tool.c
+TPM2_TOOL_REGISTER("policytemplate", tpm2_tool_onstart, tpm2_tool_onrun, tpm2_tool_onstop, NULL)
